! =================================================================================================================================
! MODULE            : stomate_io
!
! CONTACT           : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE           : IPSL (2006)
!                     This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF              Module for read and write of restart files for all stomate modules.
!!
!!\n DESCRIPTION    : This module contains the subroutines readstart and writerestart. All variables that will be read or written
!!                    are passed as argument to the subroutines. The subroutine readstart is called from stomate_initialize and 
!!                    writerestart is called from stomate_finalize.
!!                    Note: Not all variables saved in the start files are absolutely necessary. However, Sechiba's and Stomate's 
!!                    PFTs are not necessarily identical, and for that case this information needs to be saved.
!!
!!
!! RECENT CHANGE(S) : None
!!
!! REFERENCE(S)	    : None
!!
!! SVN              :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_io
  USE stomate_data
  USE constantes
  USE constantes_soil
  USE mod_orchidee_para
  USE ioipsl_para
  USE structures
  !-
  IMPLICIT NONE
  !-
  PRIVATE
  PUBLIC readstart, writerestart
  !-
  ! reference temperature (K)
  !-
  REAL(r_std),ALLOCATABLE,DIMENSION(:),SAVE :: trefe
!$OMP THREADPRIVATE(trefe)
  !-

CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : readstart
!!
!>\BRIEF          Read all variables for stomate from restart file. 
!!
!! DESCRIPTION  : Read all variables for stomate from restart file. 
!!                Initialize the variables if they were not found in the restart file or if there was no restart file.
!!                
!! \n
!_ ================================================================================================================================

  SUBROUTINE readstart &
       & (npts, index, lalo, resolution, t2m, &
       &  dt_days, date, &
       &  adapted, regenerate, &
       &  moiavail_daily, gdd_init_date, litterhum_daily, &
       &  t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
       &  soilhum_daily, precip_daily, &
       &  gpp_daily, npp_daily, turnover_daily, &
       &  moiavail_month, moiavail_week, moiavail_growingseason, &
       &  t2m_longterm, tau_longterm, t2m_month, t2m_week, &
       &  tsoil_month, soilhum_month, fireindex, firelitter, &
       &  maxmoiavail_lastyear, maxmoiavail_thisyear, &
       &  minmoiavail_lastyear, minmoiavail_thisyear, &
       &  maxgppweek_lastyear, maxgppweek_thisyear, &
       &  gdd0_lastyear, gdd0_thisyear, &
       &  precip_lastyear, precip_thisyear, &
       &  gdd_m5_dormance,  gdd_from_growthinit, gdd_midwinter, &
       &  ncd_dormance, ngd_minus5, &
       &  PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
       &  maxfpc_lastyear, maxfpc_thisyear, &
       &  turnover_longterm, gpp_week, resp_maint_part, &
       &  leaf_age, leaf_frac, plant_status, when_growthinit, age, &
       &  resp_hetero, resp_maint, resp_growth, co2_fire, atm_to_bm, &
       &  veget_lastlight, everywhere, need_adjacent, RIP_time, &
       &  time_hum_min, hum_min_dormance, &
       &  litter, dead_leaves, &
       &  som, lignin_struc, lignin_wood, turnover_time, &
       &  prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
       &  flux_prod_s, flux_prod_m, flux_prod_l, &
       &  bm_to_litter, carb_mass_total, &
       &  Tseason, Tseason_length, Tseason_tmp, & 
       &  Tmin_spring_time, onset_date, &
       &  global_years, ok_equilibrium, nbp_accu, nbp_flux, &
       &  nbp_accu_pool, nbp_pool, nbp_pool_start, &
       &  MatrixV, VectorU, previous_stock, current_stock, &
       &  assim_param, CN_som_litter_longterm, &
       &  tau_CN_longterm, KF, k_latosa_adapt, &
       &  rue_longterm, cn_leaf_min_season, nstress_season, &
       &  soil_n_min, p_O2, bact, &
       &  circ_class_biomass, circ_class_n, store_sum_delta_ba, & 
       &  forest_managed, &
       &  forest_managed_lastyear, &
       &  species_change_map, fm_change_map, lpft_replant, lai_per_level, &
       &  laieff_fit, wstress_season, wstress_month, &
       &  age_stand, rotation_n, last_cut, mai, pai, &
       &  previous_wood_volume, mai_count, coppice_dens, &
       &  light_tran_to_level_season,veget_cov_max, harvest_5y_area)

    IMPLICIT NONE

    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in) :: npts
    ! Indices of the points on the map
    INTEGER(i_std),DIMENSION(npts),INTENT(in) :: index
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: lalo
    ! size in x an y of the grid (m)
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: resolution
    REAL(r_std),DIMENSION(npts),INTENT(in)   :: t2m                !! 2 m air temperature from forcing file or coupled model (K)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: veget_cov_max       !! Maximum fraction of vegetation type including 
                                                                   !! non-biological fraction (unitless)
    !-
    ! 0.2 output
    !-
    ! time step of STOMATE in days
    REAL(r_std),INTENT(out) :: dt_days
    ! date (d)
    INTEGER(i_std),INTENT(out) :: date
    ! Winter too cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: adapted
    ! Winter sufficiently cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: regenerate
    ! daily moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_daily
    ! date for beginning of gdd count
    REAL(r_std),DIMENSION(npts,2),INTENT(out) :: gdd_init_date
    ! daily litter humidity
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: litterhum_daily
    ! daily 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_daily
    ! daily minimum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_min_daily
    ! daily surface temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: tsurf_daily
    ! daily soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: tsoil_daily
    ! daily soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: soilhum_daily
    ! daily precipitations (mm/day) (for phenology)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_daily
    ! daily gross primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gpp_daily
    ! daily net primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: npp_daily
    ! daily turnover rates (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out) :: turnover_daily
    ! "monthly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_month
    ! "weekly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_week
    ! mean growing season moisture availability (used for allocation response)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: moiavail_growingseason
    ! "long term" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_longterm
    ! "tau_longterm"
    REAL(r_std), INTENT(out)        :: tau_longterm
    ! "monthly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_month
    ! "seasonal" 2 meter temperatures (K) 
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason_length
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: Tseason_tmp
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)  :: Tmin_spring_time
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out)  :: onset_date
!!$    LOGICAL,DIMENSION(npts,nvm),INTENT(out)      :: begin_leaves
    ! "weekly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: t2m_week
    ! "monthly" soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: tsoil_month
    ! "monthly" soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(out) :: soilhum_month
    ! Probability of fire
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: fireindex
    ! Longer term total litter above the ground, gC/m**2 of ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: firelitter
    ! last year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxmoiavail_lastyear
    ! this year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxmoiavail_thisyear
    ! last year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: minmoiavail_lastyear
    ! this year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: minmoiavail_thisyear
    ! last year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxgppweek_lastyear
    ! this year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxgppweek_thisyear
    ! last year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: gdd0_lastyear
    ! this year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: gdd0_thisyear
    ! last year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_lastyear
    ! this year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(out)      :: precip_thisyear
    ! growing degree days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_m5_dormance
    ! growing degree days, from begin of season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_from_growthinit
    ! growing degree days since midwinter (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gdd_midwinter
    ! number of chilling days since leaves were lost (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: ncd_dormance
    ! number of growing days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: ngd_minus5
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs)
    LOGICAL,DIMENSION(npts,nvm),INTENT(out)    :: PFTpresent
    ! "long term" net primary productivity (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: npp_longterm
    ! last year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: lm_lastyearmax
    ! this year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: lm_thisyearmax
    ! last year's maximum fpc for each natural PFT, on ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxfpc_lastyear
    ! this year's maximum fpc for each PFT,
    ! on *total* ground (see stomate_season)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: maxfpc_thisyear
    ! "long term" turnover rate (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out) :: turnover_longterm
    ! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: gpp_week
    ! maintenance resp (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts),INTENT(out) :: resp_maint_part
    ! leaf age (days)
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(out) :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(out) :: leaf_frac
    !! Growth and phenological status of the plant
    !! The different stati are defined in constantes
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: plant_status
    ! how many days ago was the beginning of the growing season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: when_growthinit
    ! mean age (years)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: age
    ! heterotrophic respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_hetero
    ! maintenance respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_maint
    ! growth respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: resp_growth
    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2/time step)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: co2_fire
    ! biomass taken from the atmosphere (gC or gN /(m**2 of total ground)/day)
    REAL(r_std),DIMENSION(npts,nvm,nelements),INTENT(out) :: atm_to_bm
    ! vegetation fractions (on ground) after last light competition
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: veget_lastlight
    ! is the PFT everywhere in the grid box or very localized
    ! (after its introduction)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: everywhere
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box?
    LOGICAL,DIMENSION(npts,nvm),INTENT(out) :: need_adjacent
    ! How much time ago was the PFT eliminated for the last time (y)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: RIP_time
    ! time elapsed since strongest moisture availability (d)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: time_hum_min
    ! minimum moisture during dormance
    REAL(r_std),DIMENSION(npts,nvm),INTENT(out) :: hum_min_dormance
    ! fraction of litter above the ground belonging to different PFTs
    ! separated for natural and agricultural PFTs.
    REAL(r_std),DIMENSION(npts,nlitt,nvm,nlevs,nelements),INTENT(out):: litter
    ! dead leaves on ground, per PFT, metabolic and structural,
    ! in gC/(m**2 of ground)
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(out) :: dead_leaves
    ! Soil Organic Matter pool: active, slow, or passive, (gC (or N)/m**2)
    REAL(r_std),DIMENSION(npts,ncarb,nvm,nelements),INTENT(out) :: som
    ! ratio Lignine/Carbon in structural litter, above and below ground,(gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(out) :: lignin_struc
    ! ratio Lignine/Carbon in woody litter, above and below ground,(gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(out) :: lignin_wood
    REAL(r_std),DIMENSION(npts,nvm,nparts),INTENT(out) :: turnover_time

    ! For Spinup matrix resolution
    INTEGER(i_std), INTENT(out)                   :: global_years   
    LOGICAL, DIMENSION(npts), INTENT(out)         :: ok_equilibrium
    REAL(r_std), DIMENSION(npts), INTENT(out)     :: nbp_accu       !! Accumulated Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts), INTENT(out)     :: nbp_flux       !! Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out) :: nbp_accu_pool  !! Pool-based Accumulated Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out) :: nbp_pool       !! Pool-based Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out) :: nbp_pool_start !! NBP at previous time step

    !-
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(out) :: MatrixV
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out) :: VectorU
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out) :: previous_stock
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out) :: current_stock
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out) :: CN_som_litter_longterm
    REAL(r_std), INTENT(out)                              :: tau_CN_longterm  !! Counter used for calculating the longterm CN ratio of SOM and litter pools (seconds)
    REAL(r_std), DIMENSION(npts,nvm,npco2),   INTENT(out) :: assim_param

    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)           :: KF             !! Scaling factor to convert sapwood mass                     
                                                                              !! into leaf mass (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)           :: k_latosa_adapt !! Leaf to sapwood area adapted for water       
                                                                              !! stress. Adaptation takes place at the 
                                                                              !! end of the year (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: mai                     !! The mean annual increment
                                                                                                   !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: pai                     !! The period annual increment
                                                                                                   !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                                   !! in a stand for the previous year.
                                                                                                   !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(out)                     :: mai_count               !! The number of times we've
                                                                                                   !! calculated the volume increment
                                                                                                   !! for a stand
    REAL(r_std), DIMENSION(npts,nvm),INTENT(out)                        :: coppice_dens            !! The density of a coppice at the first
                                                                                                   !! cutting.
                                                                                                   !! @tex $( 1 / m**2 )$ @endtex

    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: rue_longterm            !! longterm radiation use efficiency
    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: rotation_n              !! Rotation number (number of rotation 
                                                                                                   !! since pft is managed)
    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: last_cut                !! Years since last thinning (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: cn_leaf_min_season          !! Seasonal min CN ratio of leaves 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: nstress_season          !! N-related seasonal stress (used for allocation) 
   
    REAL(r_std), DIMENSION(npts,nvm,nnspec), INTENT(out)                :: soil_n_min              !! mineral nitrogen in the soil (gN/m**2)  
                                                                                                   !! (first index=npts, second index=nvm, third index=nnspec) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: p_O2                    !! partial pressure of oxigen in the soil (hPa)
                                                                                                   !! (first index=npts, second index=nvm)
                      
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: bact                    !! denitrifier biomass (gC/m**2)
                                                                                                   !! (first index=npts, second index=nvm)

    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)                 :: circ_class_n            !! Number of trees in each circumference classes
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), INTENT(out):: circ_class_biomass      !! Biomass components of the model tree within 
                                                                                                   !! each circumference classes (gC ind^-1) 
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(inout)               :: store_sum_delta_ba      !! Sum of increase of basal area(m^2) which
                                                                                                   !! doesn't pass through gap clean 
    INTEGER(i_std), DIMENSION (npts,nvm), INTENT(out)                   :: forest_managed_lastyear !! forest management flag for the previous year
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(out)                    :: forest_managed          !! forest management flag
    REAL(r_std), DIMENSION(npts,0:nshort,nelements), INTENT(out)        :: prod_s                  !!
    REAL(r_std), DIMENSION(npts,0:nmedium,nelements), INTENT(out)       :: prod_m                  !!
    REAL(r_std), DIMENSION(npts,0:nlong,nelements), INTENT(out)         :: prod_l                  !!
    REAL(r_std), DIMENSION(npts,nshort,nelements), INTENT(out)          :: flux_s                  !!
    REAL(r_std), DIMENSION(npts,nmedium,nelements), INTENT(out)         :: flux_m                  !!
    REAL(r_std), DIMENSION(npts,nlong,nelements), INTENT(out)           :: flux_l                  !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(out)                 :: flux_prod_s            !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(out)                 :: flux_prod_m            !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(out)                 :: flux_prod_l            !!
    INTEGER(i_std), DIMENSION(:,:), INTENT(out)                         :: species_change_map      !! A map which gives the PFT number that each
                                                                                                   !! PFT will be replanted as in case of a clearcut.
                                                                                                   !! (1-nvm,unitless)
    INTEGER(i_std), DIMENSION(:,:), INTENT(out)                         :: fm_change_map           !! A map which gives the desired FM strategy when
                                                                                                   !! the PFT will be replanted after a clearcut.
                                                                                                   !! (1-nvm,unitless)
    LOGICAL, DIMENSION(:,:), INTENT(out)                                :: lpft_replant            !! Indicates if this PFT has either died this year
                                                                                                   !! or been clearcut/coppiced.  If it has, it is not
                                                                                                   !! replanted until the end of the year.


    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot),INTENT(out)            :: lai_per_level           !! The amount of LAI in each physical

   !! canopy level. 
                                                                                                   !! @tex $( m**2 / m**2 )$ @endtex
 
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(out)                     :: laieff_fit              !! Fitted parameters for the effective LAI

    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: wstress_season          !! Water stress factor, based on hum_rel_daily

                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: wstress_month           !! Water stress factor, based on hum_rel_daily
                                                                                                   !! (unitless, 0-1)

    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(out)        :: bm_to_litter
    REAL(r_std),DIMENSION(npts),INTENT(out)                             :: carb_mass_total
    REAL(r_std), DIMENSION(:,:,:), INTENT(out)                          :: light_tran_to_level_season  !! Mean seasonal fraction of light transmitted to
                                                                                                   !! a given canopy level (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(out)                          :: harvest_5y_area         !! Total harvested area in the last 5 years (m^{2})

  !! 0.4 Local variables
    
    REAL(r_std) :: date_real
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs), real
    REAL(r_std),DIMENSION(npts,nvm) :: PFTpresent_real
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box? - real
    REAL(r_std),DIMENSION(npts,nvm) :: need_adjacent_real
    REAL(r_std), DIMENSION(1) :: vartmp  !! temporary variable because restget/restput needs an array and not a scalar
    ! To store variables names for I/O
    CHARACTER(LEN=80) :: var_name
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: part_str
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: circ_str             
    ! string suffix indicating litter type
    CHARACTER(LEN=4),DIMENSION(nlitt) :: litter_str
    ! string suffix indicating level
    CHARACTER(LEN=2),DIMENSION(nlevs) :: level_str
    ! temporary storage
    REAL(r_std),DIMENSION(1) :: xtmp
    ! index
    INTEGER(i_std) :: j,k,l,m
    ! reference temperature (K)

    CHARACTER(LEN=2),DIMENSION(nelements) :: element_str   !! string suffix indicating element
    REAL(r_std), DIMENSION(1) :: temp_global_years
    CHARACTER(LEN=6), DIMENSION(nbpools) :: pools_str
    REAL(r_std), DIMENSION(npts) :: ok_equilibrium_real    
    CHARACTER(LEN=80) :: filename   !! filename for reference temerature file
    ! land cover change variables
    INTEGER                                                             :: n,ilev,ipts,ivm,icarb   !! Indices
    REAL(r_std), DIMENSION(npts,nvm)                                    :: temp_real               !! temporary real to allow restget 
                                                                                                   !! to work on multi-dimensional integers
    REAL(r_std), DIMENSION(npts,nvm)                                    :: r_replant               !! Getting logical values from the restart
                                                                                                   !! is not possible, so this is a temporary
                                                                                                   !! array where 1.0 is TRUE.
    REAL(r_std),DIMENSION(npts,nvm,nlevels_tot,nparams_laieff)          :: temp_array              !! To store structure values for I/O
    CHARACTER(LEN=10)                                                   :: part_str2               !! string suffix indicating an index

    CHARACTER(LEN=2), DIMENSION(wind_years)                             :: wyear_str               !! string suffix indicating wind year index


   !_ ================================================================================================================================

    IF (printlev >= 3) WRITE(numout,*) 'Entering readrestart'
    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSEIF (l == iwoody) THEN
          litter_str(l) = 'wood'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart', 'Define litter_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart','Define level_str','','')
       ENDIF
    ENDDO

    pools_str(1:nbpools) =(/'str_ab ','str_be ','met_ab ','met_be ','wood_ab','wood_be',& 
         & 'actif  ','slow   ','passif ','surface'/) 
    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
       ELSEIF (l == initrogen) THEN
          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p(3,'stomate_io readstart','Define element_str','','')
       ENDIF
    ENDDO
    !-
    ! 2 run control
    !-
    ! 2.2 time step of STOMATE in days
    !-
    IF (is_root_prc) THEN
       var_name = 'dt_days'
       CALL restget (rest_id_stomate, var_name, 1   , 1     , 1, itime, &
            &                 .TRUE., xtmp)
       dt_days = xtmp(1)
       IF (dt_days == val_exp) dt_days = un
    ENDIF
    CALL bcast(dt_days)
    !-
    ! 2.3 date
    !-
    IF (is_root_prc) THEN
       var_name = 'date'
       CALL restget (rest_id_stomate, var_name, 1   , 1     , 1, itime, &
            &                 .TRUE., xtmp)
       date_real = xtmp(1)
       IF (date_real == val_exp) date_real = zero
       date = NINT(date_real)
    ENDIF
    CALL bcast(date)
    !-
    ! 3 daily meteorological variables
    !-
    moiavail_daily(:,:) = val_exp
    var_name = 'moiavail_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_daily, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_daily(:,:) == val_exp)) moiavail_daily(:,:) = zero
    !-
    gdd_init_date(:,:) = val_exp
    var_name = 'gdd_init_date'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 2 , 1, itime, &
         &              .TRUE., gdd_init_date, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_init_date(:,1) == val_exp)) gdd_init_date(:,1) = 365.
    !-
    litterhum_daily(:) = val_exp
    var_name = 'litterhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., litterhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(litterhum_daily(:) == val_exp)) litterhum_daily(:) = zero
    !-
    t2m_daily(:) = val_exp
    var_name = 't2m_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_daily(:) == val_exp)) t2m_daily(:) = zero
    !-
    t2m_min_daily(:) = val_exp
    var_name = 't2m_min_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_min_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_min_daily(:) == val_exp)) t2m_min_daily(:) = large_value
    !-
    tsurf_daily(:) = val_exp
    var_name = 'tsurf_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., tsurf_daily, 'gather', nbp_glo, index_g)
    ! The initial value is set to the current temperature at 2m
    IF (ALL(tsurf_daily(:) == val_exp)) tsurf_daily(:) = t2m(:)
    !-
    tsoil_daily(:,:) = val_exp
    var_name = 'tsoil_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., tsoil_daily, 'gather', nbp_glo, index_g)
    IF (ALL(tsoil_daily(:,:) == val_exp)) tsoil_daily(:,:) = zero
    !-
    soilhum_daily(:,:) = val_exp
    var_name = 'soilhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., soilhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_daily(:,:) == val_exp)) soilhum_daily(:,:) = zero
    !-
    precip_daily(:) = val_exp
    var_name = 'precip_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., precip_daily, 'gather', nbp_glo, index_g)
    IF (ALL(precip_daily(:) == val_exp)) precip_daily(:) = zero
    !-
    ! 4 productivities
    !-
    gpp_daily(:,:) = val_exp
    var_name = 'gpp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_daily(:,:) == val_exp)) gpp_daily(:,:) = zero
    !-
    npp_daily(:,:) = val_exp
    var_name = 'npp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(npp_daily(:,:) == val_exp)) npp_daily(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'turnover_daily', nbp_glo, nvm, nparts, nelements, itime, & 
        &                .TRUE., turnover_daily, 'gather', nbp_glo, index_g) 
    IF (ALL(turnover_daily  == val_exp)) turnover_daily(:,:,:,:) = zero
    !-
    ! 5 monthly meteorological variables
    !-
    moiavail_month(:,:) = val_exp
    var_name = 'moiavail_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_month, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_month(:,:) == val_exp)) moiavail_month(:,:) = zero
    !-
    moiavail_week(:,:) = val_exp
    var_name = 'moiavail_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_week, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_week(:,:) == val_exp)) moiavail_week(:,:) = zero

    moiavail_growingseason(:,:) = val_exp
    var_name = 'moiavail_grow'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_growingseason, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_growingseason(:,:) == val_exp)) moiavail_growingseason(:,:) = un
    

    !
    ! Longterm temperature at 2m
    !
    var_name = 't2m_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_longterm, 'gather', nbp_glo, index_g)

    IF (ALL(t2m_longterm(:) == val_exp)) THEN
       ! t2m_longterm is not in restart file
       ! The initial value for the reference temperature is set to the current temperature
       t2m_longterm(:)=t2m(:)
       ! Set the counter to 2 time steps
       tau_longterm=2
    ELSE
       ! t2m_longterm was in the restart file
       ! Now read tau_longterm
       ! tau_longterm is a scalar, therefor only master process read this value
       IF (is_root_prc) THEN
          CALL restget (rest_id_stomate, 'tau_longterm', 1 ,1  , 1, itime, &
               .TRUE., vartmp)
          IF (vartmp(1) == val_exp) THEN
             ! tau_longterm is not found in restart file. 
             ! This is not normal as t2m_longterm was in restart file. Write a warning and initialize it to tau_longterm_max
             CALL ipslerr(2, 'stomate_io readstart','tau_longterm was not in restart file',&
                  'But t2m_longterm was in restart file','')
             tau_longterm = tau_longterm_max
          ELSE
             tau_longterm = vartmp(1)
          END IF
       ENDIF
       CALL bcast(tau_longterm)

    END IF
    !-
    t2m_month(:) = val_exp
    var_name = 't2m_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_month, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_month(:) == val_exp)) t2m_month(:) = t2m(:)
    
    CALL restget_p (rest_id_stomate, 'Tseason', nbp_glo, 1     , 1, itime, &
         .TRUE., Tseason, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason(:) == val_exp)) Tseason(:) = t2m(:)
    
    CALL restget_p (rest_id_stomate,'Tseason_length', nbp_glo, 1     , 1, itime, &
         .TRUE., Tseason_length, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason_length(:) == val_exp)) Tseason_length(:) = zero
    
    CALL restget_p (rest_id_stomate, 'Tseason_tmp', nbp_glo, 1     , 1, itime, &
         .TRUE., Tseason_tmp, 'gather', nbp_glo, index_g)
    IF (ALL(Tseason_tmp(:) == val_exp)) Tseason_tmp(:) = zero

    CALL restget_p (rest_id_stomate, 'Tmin_spring_time', nbp_glo, nvm, 1, itime, &
         .TRUE., Tmin_spring_time, 'gather', nbp_glo, index_g)
    IF (ALL(Tmin_spring_time(:,:) == val_exp)) Tmin_spring_time(:,:) = zero
    
    CALL restget_p (rest_id_stomate, 'onset_date', nbp_glo, nvm  , 1, itime, &
         .TRUE., onset_date(:,:), 'gather', nbp_glo, index_g)
    IF (ALL(onset_date(:,:) == val_exp)) onset_date(:,:) = zero

    t2m_week(:) = val_exp
    var_name = 't2m_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_week, 'gather', nbp_glo, index_g)
    ! The initial value is set to the current temperature
    IF (ALL(t2m_week(:) == val_exp)) t2m_week(:) = t2m(:)
    
    tsoil_month(:,:) = val_exp
    var_name = 'tsoil_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., tsoil_month, 'gather', nbp_glo, index_g)

    ! The initial value is set to the current temperature
    IF (ALL(tsoil_month(:,:) == val_exp)) THEN
       DO l=1,nbdl
          tsoil_month(:,l) = t2m(:)
       ENDDO
    ENDIF
    !-
    soilhum_month(:,:) = val_exp
    var_name = 'soilhum_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., soilhum_month, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_month(:,:) == val_exp)) soilhum_month(:,:) = zero
    !-
    wstress_season(:,:) = val_exp
    var_name = 'wstress_season'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., wstress_season, 'gather', nbp_glo, index_g)
    IF (ALL(wstress_season(:,:) == val_exp)) wstress_season(:,:) = un
    !-
    wstress_month(:,:) = val_exp
    var_name = 'wstress_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., wstress_month, 'gather', nbp_glo, index_g)
    IF (ALL(wstress_month(:,:) == val_exp)) wstress_month(:,:) = un
    !-
    ! 6 fire probability
    !-
    fireindex(:,:) = val_exp
    var_name = 'fireindex'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., fireindex, 'gather', nbp_glo, index_g)
    IF (ALL(fireindex(:,:) == val_exp)) fireindex(:,:) = zero
    !-
    firelitter(:,:) = val_exp
    var_name = 'firelitter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., firelitter, 'gather', nbp_glo, index_g)
    IF (ALL(firelitter(:,:) == val_exp)) firelitter(:,:) = zero
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    maxmoiavail_lastyear(:,:) = val_exp
    var_name = 'maxmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_lastyear(:,:) == val_exp)) &
         &     maxmoiavail_lastyear(:,:) = zero
    !-
    maxmoiavail_thisyear(:,:) = val_exp
    var_name = 'maxmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_thisyear(:,:) == val_exp)) &
         &     maxmoiavail_thisyear(:,:) = zero
    !-
    minmoiavail_lastyear(:,:) = val_exp
    var_name = 'minmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(minmoiavail_lastyear(:,:) == val_exp)) &
         &     minmoiavail_lastyear(:,:) = un
    !-
    minmoiavail_thisyear(:,:) = val_exp
    var_name = 'minmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL( minmoiavail_thisyear(:,:) == val_exp)) &
         &     minmoiavail_thisyear(:,:) = un
    !-
    ! 8 maximum "weekly" GPP
    !-
    maxgppweek_lastyear(:,:) = val_exp
    var_name = 'maxgppweek_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_lastyear(:,:) == val_exp)) &
         &     maxgppweek_lastyear(:,:) = zero
    !-
    maxgppweek_thisyear(:,:) = val_exp
    var_name = 'maxgppweek_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_thisyear(:,:) == val_exp)) &
         &     maxgppweek_thisyear(:,:) = zero
    !-
    ! 9 annual GDD0
    !-
    gdd0_thisyear(:) = val_exp
    var_name = 'gdd0_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_thisyear(:) == val_exp)) gdd0_thisyear(:) = zero
    !-
    gdd0_lastyear(:) = val_exp
    var_name = 'gdd0_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_lastyear(:) == val_exp)) gdd0_lastyear(:) = gdd_crit_estab
    !-
    ! 10 annual precipitation
    !-
    precip_thisyear(:) = val_exp
    var_name = 'precip_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_thisyear(:) == val_exp)) precip_thisyear(:) = zero
    !-
    precip_lastyear(:) = val_exp
    var_name = 'precip_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_lastyear(:) == val_exp)) &
         &     precip_lastyear(:) = precip_crit
    !-
    ! 11 derived "biometeorological" variables
    !-
    gdd_m5_dormance(:,:) = val_exp
    var_name = 'gdd_m5_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_m5_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_m5_dormance(:,:) == val_exp)) &
         &     gdd_m5_dormance(:,:) = undef
    !-
    gdd_from_growthinit(:,:) = val_exp
    var_name = 'gdd_from_growthinit'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_from_growthinit, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_from_growthinit(:,:) == val_exp)) &
         &     gdd_from_growthinit(:,:) = zero
    !-
    gdd_midwinter(:,:) = val_exp
    var_name = 'gdd_midwinter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_midwinter, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_midwinter(:,:) == val_exp)) gdd_midwinter(:,:) = undef
    !-
    ncd_dormance(:,:) = val_exp
    var_name = 'ncd_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ncd_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(ncd_dormance(:,:) == val_exp)) ncd_dormance(:,:) = undef
    !-
    ngd_minus5(:,:) = val_exp
    var_name = 'ngd_minus5'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ngd_minus5, 'gather', nbp_glo, index_g)
    IF (ALL(ngd_minus5(:,:) == val_exp)) ngd_minus5(:,:) = zero
    !-
    time_hum_min(:,:) = val_exp
    var_name = 'time_hum_min'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., time_hum_min, 'gather', nbp_glo, index_g)
    IF (ALL(time_hum_min(:,:) == val_exp)) time_hum_min(:,:) = undef
    !-
    hum_min_dormance(:,:) = val_exp
    var_name = 'hum_min_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., hum_min_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(hum_min_dormance(:,:) == val_exp)) &
         &     hum_min_dormance(:,:) = undef
    !-
    ! 12 Plant status
    !-
    CALL restget_p (rest_id_stomate, 'PFTpresent', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., PFTpresent_real, 'gather', nbp_glo, index_g)
    IF (ALL(PFTpresent_real(:,:) == val_exp)) PFTpresent_real(:,:) = zero
    WHERE (PFTpresent_real(:,:) >= .5)
       PFTpresent = .TRUE.
    ELSEWHERE
       PFTpresent = .FALSE.
    ENDWHERE
    !-
    !CALL restget_p (rest_id_stomate, 'ind', nbp_glo, nvm  , 1, itime, &
    !     &              .TRUE., ind, 'gather', nbp_glo, index_g)
    !IF (ALL(ind(:,:) == val_exp)) ind(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'adapted', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., adapted, 'gather', nbp_glo, index_g)
    IF (ALL(adapted(:,:) == val_exp)) adapted(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'regenerate', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., regenerate, 'gather', nbp_glo, index_g)
    IF (ALL(regenerate(:,:) == val_exp)) regenerate(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'npp_longterm', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_longterm, 'gather', nbp_glo, index_g)
    IF (ALL(npp_longterm(:,:) == val_exp)) npp_longterm(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'lm_lastyearmax', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_lastyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_lastyearmax(:,:) == val_exp)) lm_lastyearmax(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'lm_thisyearmax', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_thisyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_thisyearmax(:,:) == val_exp)) lm_thisyearmax(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'maxfpc_lastyear', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_lastyear(:,:) == val_exp)) maxfpc_lastyear(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'maxfpc_thisyear', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_thisyear(:,:) == val_exp)) maxfpc_thisyear(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'turnover_time', nbp_glo, nvm  , nparts, itime, &
         &              .TRUE., turnover_time, 'gather', nbp_glo, index_g)
    IF ( ALL( turnover_time(:,:,:) == val_exp)) turnover_time(:,:,:) = 100.
    !-
    CALL restget_p (rest_id_stomate, 'turnover_longterm', nbp_glo, nvm, nparts, nelements, itime, & 
         &              .TRUE., turnover_longterm, 'gather', nbp_glo, index_g) 
    IF (ALL(turnover_longterm == val_exp)) turnover_longterm(:,:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'gpp_week', nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_week, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_week(:,:) == val_exp)) gpp_week(:,:) = zero
    !-
    !CALL restget_p (rest_id_stomate, 'biomass', nbp_glo, nvm, nparts, nelements, itime, &
    !     &              .TRUE., biomass, 'gather', nbp_glo, index_g)
    !IF (ALL(biomass == val_exp)) biomass(:,:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'circ_class_n', nbp_glo, nvm, ncirc, itime, &
         &              .TRUE., circ_class_n, 'gather', nbp_glo, index_g)
    IF (ALL(circ_class_n == val_exp)) circ_class_n(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'store_sum_delta_ba', nbp_glo, nvm, ncirc, itime, &
         &              .TRUE., store_sum_delta_ba, 'gather', nbp_glo, index_g)
    IF (ALL(store_sum_delta_ba == val_exp)) store_sum_delta_ba(:,:,:) = zero
    !-
    DO m = 1,nelements
        var_name = 'cc_biomass_'//TRIM(element_str(m))
        CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, ncirc, nparts, itime, &
             &         .TRUE., circ_class_biomass(:,:,:,:,m), 'gather', nbp_glo, index_g)
        IF (ALL(circ_class_biomass(:,:,:,:,m) == val_exp)) circ_class_biomass(:,:,:,:,m) = zero
    ENDDO
    !-
    CALL restget_p (rest_id_stomate, 'lai_per_level', nbp_glo, nvm, nlevels_tot, itime, &
         &               .TRUE., lai_per_level, 'gather', nbp_glo, index_g)
    IF (ALL(lai_per_level == val_exp)) lai_per_level(:,:,:) = zero
    !
    ! This is ugly, since the restart routines were not developed to deal with structures. 
    ! But all I have to do is copy some things to arrays.
    temp_array(:,:,:,:) = val_exp
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       DO n=1,nparams_laieff
          WRITE(part_str2,'(I2)') n
          IF ( n < 10 ) part_str2(1:1) = '0'

          var_name = 'laieff_fit_'//part_str(1:LEN_TRIM(part_str)) // '_' &
               // part_str2(1:LEN_TRIM(part_str2))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               .TRUE., temp_array(:,:,m,n), 'gather', nbp_glo, index_g)
          IF (ALL(temp_array(:,:,m,n) == val_exp)) temp_array(:,:,m,n) = zero
       ENDDO
    ENDDO
    DO ipts=1,npts
       DO ivm=1,nvm
          DO ilev=1,nlevels_tot
             laieff_fit(ipts,ivm,ilev)%a=temp_array(ipts,ivm,ilev,1)
             laieff_fit(ipts,ivm,ilev)%b=temp_array(ipts,ivm,ilev,2)
             laieff_fit(ipts,ivm,ilev)%c=temp_array(ipts,ivm,ilev,3)
             laieff_fit(ipts,ivm,ilev)%d=temp_array(ipts,ivm,ilev,4)
             laieff_fit(ipts,ivm,ilev)%e=temp_array(ipts,ivm,ilev,5)
          ENDDO
       ENDDO
    ENDDO
    !
    CALL restget_p (rest_id_stomate, 'maint_resp', nbp_glo, nvm, nparts, itime, &
         &                   .TRUE., resp_maint_part, 'gather', nbp_glo, index_g)
    IF (ALL(resp_maint_part == val_exp)) resp_maint_part(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'leaf_age', nbp_glo, nvm, nleafages, itime, &
         &                   .TRUE., leaf_age(:,:,:), 'gather', nbp_glo, index_g)
    IF (ALL(leaf_age == val_exp)) leaf_age(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'leaf_frac', nbp_glo, nvm, nleafages, itime, &
         &                  .TRUE., leaf_frac(:,:,:), 'gather', nbp_glo, index_g)
    IF (ALL(leaf_frac(:,:,:) == val_exp)) leaf_frac(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'plant_status', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., plant_status, 'gather', nbp_glo, index_g)
    IF (ALL(plant_status(:,:) == val_exp)) THEN
       plant_status(:,:) = iprescribe
       plant_status(:,ibare_sechiba) = inone
    ENDIF
    !-
    CALL restget_p (rest_id_stomate, 'when_growthinit', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., when_growthinit, 'gather', nbp_glo, index_g)
    IF (ALL(when_growthinit(:,:) == val_exp)) when_growthinit(:,:) = 240.
    !-
    CALL restget_p (rest_id_stomate, 'age', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., age, 'gather', nbp_glo, index_g)
    IF (ALL(age(:,:) == val_exp)) age(:,:) = zero
    !-
    ! 13 CO2
    !-
    CALL restget_p (rest_id_stomate, 'resp_hetero', nbp_glo, nvm, 1, itime, &
         &                .TRUE., resp_hetero, 'gather', nbp_glo, index_g)
    IF (ALL(resp_hetero(:,:) == val_exp)) resp_hetero(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'resp_maint', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_maint, 'gather', nbp_glo, index_g)
    IF (ALL(resp_maint(:,:) == val_exp)) resp_maint(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'resp_growth', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_growth, 'gather', nbp_glo, index_g)
    IF (ALL(resp_growth(:,:) == val_exp)) resp_growth(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'co2_fire', nbp_glo, nvm     , 1, itime, &
         &                .TRUE., co2_fire, 'gather', nbp_glo, index_g)
    IF (ALL(co2_fire(:,:) == val_exp)) co2_fire(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'atm_to_bm', nbp_glo, nvm, nelements, itime, &
         &                .TRUE., atm_to_bm, 'gather', nbp_glo, index_g)
    IF (ALL(atm_to_bm == val_exp)) atm_to_bm(:,:,:) = zero
    !-
 
    !-
    ! 14 vegetation distribution after last light competition
    !-
    var_name = 'veget_lastlight'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., veget_lastlight, 'gather', nbp_glo, index_g)
    IF (ALL(veget_lastlight(:,:) == val_exp)) veget_lastlight(:,:) = zero
    !-
    ! 15 establishment criteria
    !-
    var_name = 'everywhere'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., everywhere, 'gather', nbp_glo, index_g)
    IF (ALL(everywhere(:,:) == val_exp)) everywhere(:,:) = zero
    !-
    var_name = 'need_adjacent'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., need_adjacent_real, 'gather', nbp_glo, index_g)
    IF (ALL(need_adjacent_real(:,:) == val_exp)) &
         &     need_adjacent_real(:,:) = zero
    WHERE ( need_adjacent_real(:,:) >= .5 )
       need_adjacent = .TRUE.
    ELSEWHERE
       need_adjacent = .FALSE.
    ENDWHERE
    !-
    var_name = 'RIP_time'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., RIP_time, 'gather', nbp_glo, index_g)
    IF (ALL(RIP_time(:,:) == val_exp)) RIP_time(:,:) = large_value
    !-
    ! 17 litter
    !-
    DO k = 1,nelements
       var_name = 'litter_'//element_str(k)
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nlitt, nvm, nlevs, itime, &
            &                     .TRUE., litter(:,:,:,:,k), 'gather', nbp_glo, index_g)
       IF (ALL(litter(:,:,:,:,k) == val_exp)) litter(:,:,:,:,k) = zero
    END DO
    !-
    CALL restget_p (rest_id_stomate, 'dead_leaves', nbp_glo, nvm, nlitt, itime, &
          &                   .TRUE., dead_leaves, 'gather', nbp_glo, index_g)
    IF (ALL(dead_leaves == val_exp)) dead_leaves(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'soil_carbon', nbp_glo, ncarb, nvm, itime, &
         &                   .TRUE., som(:,:,:,icarbon), 'gather', nbp_glo, index_g) 
    ! Initialise the soil pools. 
    IF (ALL(som(:,:,:,icarbon) == val_exp)) THEN
       ! We set the pools to zero for all PFTs
       som(:,:,:,icarbon) = zero
       DO ipts =1,npts
          DO ivm = 1,nvm 
             ! Set the initial values for PFTs that are present.  
             IF( veget_cov_max(ipts,ivm) .GT. min_stomate) THEN 
                som(ipts,iactive,ivm,icarbon) = som_init_active 
                som(ipts,isurface,ivm,icarbon) = som_init_surface 
                som(ipts,islow,ivm,icarbon) = som_init_slow
                som(ipts,ipassive,ivm,icarbon) = som_init_passive
             ENDIF
          ENDDO
       ENDDO  
    ENDIF
    !-
    CALL restget_p (rest_id_stomate, 'soil_nitrogen', nbp_glo, ncarb, nvm, itime, & 
         &                   .TRUE., som(:,:,:,initrogen), 'gather', nbp_glo, index_g) 
    IF (ALL(som(:,:,:,initrogen) == val_exp)) THEN 
       som(:,iactive,:,initrogen) = som(:,iactive,:,icarbon) / CN_target_iactive_ref 
       som(:,isurface,:,initrogen) = som(:,isurface,:,icarbon) / CN_target_isurface_ref 
       som(:,islow,:,initrogen) = som(:,islow,:,icarbon) / CN_target_islow_ref 
       som(:,ipassive,:,initrogen) =  som(:,ipassive,:,icarbon) / CN_target_ipassive_ref 
    ENDIF
    !-
    CALL restget_p (rest_id_stomate, 'lignin_struc', nbp_glo, nvm, nlevs, itime, &
         &     .TRUE., lignin_struc, 'gather', nbp_glo, index_g)
    IF (ALL(lignin_struc == val_exp)) lignin_struc(:,:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'lignin_wood', nbp_glo, nvm, nlevs, itime, &
         &     .TRUE., lignin_wood, 'gather', nbp_glo, index_g)
    IF (ALL(lignin_wood == val_exp)) lignin_wood(:,:,:) = zero
    !-
    ! 18 Product use and LCC
    !-
    CALL restget_p (rest_id_stomate, 'prod_s', nbp_glo, nshort+1, nelements, itime, &
         &                .TRUE., prod_s, 'gather', nbp_glo, index_g)
    IF (ALL(prod_s == val_exp)) prod_s(:,:,:) = zero
        
    CALL restget_p (rest_id_stomate, 'prod_m', nbp_glo, nmedium+1, nelements, itime, &
         &                .TRUE., prod_m, 'gather', nbp_glo, index_g)
    IF (ALL(prod_m == val_exp)) prod_m(:,:,:) = zero

    CALL restget_p (rest_id_stomate, 'prod_l', nbp_glo, nlong+1, nelements, itime, &
         &                .TRUE., prod_l, 'gather', nbp_glo, index_g)
    IF (ALL(prod_l == val_exp)) prod_l(:,:,:) = zero

    CALL restget_p (rest_id_stomate, 'flux_s', nbp_glo, nshort , nelements, itime, &
         &                .TRUE., flux_s, 'gather', nbp_glo, index_g)
    IF (ALL(flux_s == val_exp)) flux_s(:,:,:) = zero
        
    CALL restget_p (rest_id_stomate, 'flux_m', nbp_glo, nmedium, nelements, itime, &
         &                .TRUE., flux_m, 'gather', nbp_glo, index_g)
    IF (ALL(flux_m == val_exp)) flux_m(:,:,:) = zero

    CALL restget_p (rest_id_stomate, 'flux_l', nbp_glo, nlong, nelements, itime, &
         &                .TRUE., flux_l, 'gather', nbp_glo, index_g)
    IF (ALL(flux_l == val_exp)) flux_l(:,:,:) = zero

    CALL restget_p (rest_id_stomate, 'flux_prod_s', nbp_glo, nelements, 1, itime, &
         &              .TRUE., flux_prod_s, 'gather', nbp_glo, index_g)
    IF (ALL(flux_prod_s == val_exp)) flux_prod_s(:,:) = zero

    CALL restget_p (rest_id_stomate, 'flux_prod_m', nbp_glo, nelements, 1, itime, &
         &              .TRUE., flux_prod_m, 'gather', nbp_glo, index_g)
    IF (ALL(flux_prod_m == val_exp)) flux_prod_m(:,:) = zero

    CALL restget_p (rest_id_stomate, 'flux_prod_l', nbp_glo, nelements, 1, itime, &
         &              .TRUE., flux_prod_l, 'gather', nbp_glo, index_g)
    IF (ALL(flux_prod_l == val_exp)) flux_prod_l(:,:) = zero
    !-
    CALL restget_p (rest_id_stomate, 'bm_to_litter', nbp_glo, nvm, nparts, nelements, itime, &
         &                .TRUE., bm_to_litter, 'gather', nbp_glo, index_g)
    IF (ALL(bm_to_litter == val_exp)) bm_to_litter(:,:,:,:) = zero
    !-
    carb_mass_total(:) = val_exp
    var_name = 'carb_mass_total'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., carb_mass_total, 'gather', nbp_glo, index_g)
    IF (ALL(carb_mass_total(:) == val_exp)) carb_mass_total(:) = zero
    !-
    ! 19. Spinup
    !-
    IF (spinup_analytic) THEN

       IF (is_root_prc) THEN
          temp_global_years(1) = val_exp
          var_name = 'Global_years'
          CALL restget (rest_id_stomate, var_name, 1 ,1  , 1, itime, &
               &                .TRUE., temp_global_years)
          IF(temp_global_years(1) == val_exp) temp_global_years(1) = zero
          global_years = INT(temp_global_years(1))
       ENDIF
       CALL bcast(global_years)

       nbp_accu(:) = val_exp
       var_name = 'nbp_sum'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_accu, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_accu(:) == val_exp)) nbp_accu(:) = zero    

       nbp_flux(:) = val_exp
       var_name = 'nbp_flux'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_flux, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_flux(:) == val_exp)) nbp_flux(:) = zero     

       nbp_pool(:,:) = val_exp
       var_name = 'nbp_pool'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
            &              .TRUE., nbp_pool(:,:), 'gather', nbp_glo, index_g)
       IF (ALL(nbp_pool(:,:) == val_exp)) nbp_pool(:,:) = zero

       nbp_pool_start(:,:) = val_exp
       var_name = 'nbp_pool_start'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
            &              .TRUE., nbp_pool_start(:,:), 'gather', nbp_glo, index_g)

       IF (ALL(nbp_pool_start(:,:) == val_exp))THEN
           nbp_pool_start(:,:) = zero
           DO ipts =1,npts
             DO ivm = 1,nvm
                ! Set the initial valies of the NBP pool. 
                ! When runnig from scratch there is neither litter og bimass
                ! pools yet, but the soil pools are set by initial values. Thus,
                ! we start by initialising the NPB pool with the soilpool.
                ! However, this means that the first year of NBP pools should be
                ! disregarded. 
                IF( veget_cov_max(ipts,ivm) .GT. min_stomate) THEN

                   DO icarb = 1, ncarb
                      nbp_pool_start(ipts,ivm) = nbp_pool_start(ipts,ivm) + &
                          som(ipts,icarb,ivm,icarbon)*veget_cov_max(ipts,ivm)
                   ENDDO
                ENDIF
             ENDDO  
           ENDDO
       ENDIF

       nbp_accu_pool(:,:) = val_exp
       var_name = 'nbp_accu_pool'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
            &              .TRUE., nbp_accu_pool, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_accu_pool(:,:) == val_exp)) nbp_accu_pool(:,:) = zero

       !-
       ok_equilibrium_real(:) = val_exp
       var_name = 'ok_equilibrium'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo , 1  , 1, itime, &
            &                .TRUE., ok_equilibrium_real,'gather', nbp_glo, index_g)
       IF (ALL(ok_equilibrium_real(:) == val_exp)) ok_equilibrium_real(:) = zero
       WHERE(ok_equilibrium_real(:) >= 0.5) 
          ok_equilibrium = .TRUE.
       ELSEWHERE
          ok_equilibrium = .FALSE.
       ENDWHERE
       !-
       MatrixV(:,:,:,:) = val_exp
       DO k = 1,nbpools
          DO j = 1,nbpools
             WRITE(part_str,'(I2)') k
             IF (k < 10) part_str(1:1) = '0'
             var_name ='MatrixV_'//part_str(1:LEN_TRIM(part_str))//'_'//TRIM(pools_str(j))
             CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime,&
                  &                     .TRUE., MatrixV(:,:,k,j), 'gather',nbp_glo, index_g)
          ENDDO
       ENDDO
       ! If nothing is found in the restart file, we initialize each submatrix
       ! by identity
       IF (ALL(MatrixV(:,:,:,:) == val_exp))  THEN
          MatrixV(:,:,:,:) = zero
          DO l = 1,nbpools
             MatrixV(:,:,l,l) = un
          END DO
       END IF


       VectorU(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'Vector_U_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., VectorU(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(VectorU(:,:,k) == val_exp))  VectorU(:,:,k) = zero
       ENDDO

       previous_stock(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'previous_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., previous_stock(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(previous_stock(:,:,k) == val_exp))  previous_stock(:,:,k) = undef_sechiba
       ENDDO

       current_stock(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'current_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., current_stock(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(current_stock(:,:,k) == val_exp))  current_stock(:,:,k) = zero
       ENDDO

       CN_som_litter_longterm(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'CN_longterm_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., CN_som_litter_longterm(:,:,k), 'gather', nbp_glo,index_g)
          IF (ALL(CN_som_litter_longterm(:,:,k) == val_exp))CN_som_litter_longterm(:,:,k) = zero
       ENDDO

       vartmp(1) = val_exp
       var_name = 'tau_CN_longterm'
       IF (is_root_prc) THEN
          CALL restget (rest_id_stomate, var_name, 1 ,1  , 1, itime, &
               .TRUE., vartmp)
          IF (vartmp(1) == val_exp) THEN
             tau_CN_longterm = dt_sechiba/one_day
          ELSE
             tau_CN_longterm = vartmp(1)
          ENDIF   
       ENDIF
       CALL bcast(tau_CN_longterm)

    ENDIF ! spinup_matrix_method

    KF(:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'KF', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., KF, 'gather', nbp_glo, index_g)
    ! I don't want to set it equal to zero, since this is a problem if these
    ! values are not here!  Better it blows up later on
    !IF (ALL(KF(:,:) == val_exp)) KF(:,:) = zero

    k_latosa_adapt(:,:) = val_exp
    CALL restget_p (rest_id_stomate, 'k_latosa_adapt', nbp_glo, nvm  , 1, itime, &
         &                .TRUE., k_latosa_adapt, 'gather', nbp_glo, index_g)
    DO m = 1,nvm
       IF (ALL(k_latosa_adapt(:,m) == val_exp)) k_latosa_adapt(:,m) = k_latosa_min(m)
    ENDDO

    rue_longterm(:,:) = val_exp
    var_name = 'rue_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., rue_longterm(:,:), 'gather', nbp_glo, index_g)
    IF (ALL(rue_longterm(:,:) == val_exp)) rue_longterm(:,:) = 1.

    cn_leaf_min_season(:,:) = val_exp 
    var_name = 'cn_leaf_min_season' 
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, & 
         &              .TRUE., cn_leaf_min_season, 'gather', nbp_glo, index_g) 
    IF ( ALL(cn_leaf_min_season(:,:) == val_exp) ) THEN 
       DO m=1,nvm 
          cn_leaf_min_season(:,m) = cn_leaf_init(m) 
       ENDDO
    ENDIF
    
    nstress_season(:,:) = val_exp 
    var_name = 'nstress_season' 
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, & 
         &              .TRUE., nstress_season, 'gather', nbp_glo, index_g) 
    IF ( ALL(nstress_season(:,:) == val_exp) ) nstress_season(:,:)=1.0 
    
    CALL restget_p (rest_id_stomate, 'soil_n_min', nbp_glo, nvm, nnspec, itime, & 
         &              .TRUE., soil_n_min, 'gather', nbp_glo, index_g) 
!    IF ( ALL(soil_n_min == val_exp) ) soil_n_min(:,:,:)=100000000. 
    IF ( ALL(soil_n_min == val_exp) ) soil_n_min(:,:,:)=0. 

    p_O2(:,:) = val_exp 
    var_name = 'p_O2'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, & 
         &              .TRUE., p_O2(:,:), 'gather', nbp_glo, index_g) 
    IF ( ALL(p_O2(:,:) == val_exp) ) p_O2(:,:)=200

    bact(:,:) = val_exp 
    var_name = 'bact'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, & 
         &              .TRUE., bact(:,:), 'gather', nbp_glo, index_g) 
    IF ( ALL(bact(:,:) == val_exp) ) bact(:,:)=10 
    !-
    temp_real(:,:) = val_exp
    var_name = 'age_stand'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
    IF ( ALL(temp_real(:,:) == val_exp) ) THEN
       age_stand(:,:) = 0
    ELSE
       age_stand = NINT(temp_real)
    ENDIF
    !-
    temp_real(:,:) = val_exp
    var_name = 'rotation_n'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
    IF ( ALL(temp_real(:,:) == val_exp) ) THEN
       rotation_n(:,:) = 1
    ELSE
       rotation_n(:,:) = NINT(temp_real(:,:))
    ENDIF
    !-
    temp_real(:,:) = val_exp
    var_name = 'last_cut'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
    IF ( ALL(temp_real(:,:) == val_exp) ) THEN
       last_cut(:,:) = 0
    ELSE
       last_cut(:,:) = NINT(temp_real(:,:))
    ENDIF
    !-
    mai(:,:) = val_exp
    var_name = 'mai'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., mai, 'gather', nbp_glo, index_g)
    IF ( ALL(mai(:,:) == val_exp) ) mai(:,:) = zero
    !-
    pai(:,:) = val_exp
    var_name = 'pai'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., pai, 'gather', nbp_glo, index_g)
    IF ( ALL(pai(:,:) == val_exp) ) pai(:,:) = zero
    !-
    previous_wood_volume(:,:) = val_exp
    var_name = 'previous_wood_volume'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., previous_wood_volume, 'gather', nbp_glo, index_g)
    IF ( ALL(previous_wood_volume(:,:) == val_exp) ) previous_wood_volume(:,:) = zero
    !-
    temp_real(:,:) = val_exp
    var_name = 'mai_count'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
    IF ( ALL(temp_real(:,:) == val_exp) ) THEN
       mai_count(:,:) = 0
    ELSE
       mai_count = NINT(temp_real)
    ENDIF
    !-
    coppice_dens(:,:) = val_exp
    var_name = 'coppice_dens'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., coppice_dens, 'gather', nbp_glo, index_g)
    IF ( ALL(coppice_dens(:,:) == val_exp) ) coppice_dens(:,:) = zero
    !-
    temp_real(:,:) = val_exp
    var_name = 'fm_lyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
    IF ( ALL(temp_real(:,:) == val_exp) ) THEN
       forest_managed_lastyear(:,:) = 0
    ELSE
       forest_managed_lastyear = NINT(temp_real)
    ENDIF

    IF (.NOT.ok_change_species) THEN
       ! In the abscence of a ok_change_species the model has to be
       ! driven by a prescribed value for forest_managed. This
       ! prescribed value can come from either a FMmap which is 
       ! read in or a value for ::FOREST_MANAGED_FORCED. In these 
       ! cases the values in the restart file are ignored.  
    ELSE
       ! We have a ok_change_species which implies that we only want
       ! to read the FMmap once at the start of the simulation
       ! after which forest_managed becomes a prognostic variable 
       ! (as it may change after a clear cut) and therefore the 
       ! values from the maps are ignored and the value from the 
       ! restart is used except at the first time step.
       temp_real(:,:) = val_exp
       var_name = 'fm_current'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          ! This is the first time step. The restart is still empty
          ! so use the values from the FMmap as read in stomate.f90
          ! Nothing should be done
       ELSE
          ! As soon as there is something in the restart file these
          ! values are used to overwrite the values that were read 
          ! from the FMmap.
          forest_managed = NINT(temp_real)
       ENDIF
       
    END IF
    
    ! The routines don't like to store integer variables.  So instead
    ! we create a real array.  If the value of the real array is one,
    ! we assign a value of TRUE to the logical array.  Anything
    ! else is .FALSE.
    r_replant(:,:) = val_exp
    var_name = 'sc_map'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF( r_replant(ipts,ivm) .LT. nvm*2)THEN
             species_change_map(ipts,ivm)=NINT(r_replant(ipts,ivm))
          ELSE
             species_change_map(ipts,ivm)=0
          ENDIF
       ENDDO
    ENDDO

    r_replant(:,:) = val_exp
    var_name = 'fmc_map'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF( r_replant(ipts,ivm) .LT. nvm*2)THEN
             fm_change_map(ipts,ivm)=NINT(r_replant(ipts,ivm))
          ELSE
             fm_change_map(ipts,ivm)=0
          ENDIF
       ENDDO
    ENDDO

    ! read harvest_5y_area from restart file
    harvest_5y_area(:,:,:) = val_exp
    DO l=1,wind_years
       WRITE(wyear_str(l),'(I2.2)') l
       var_name = 'harvest_area_year_'//wyear_str(l)
       !WRITE(numout,*) 'YC: 20160531: var_name for restart file', var_name
       CALL restget_p &
            &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &     .TRUE., harvest_5y_area(:,:,l), 'gather', nbp_glo,index_g)
       IF (ALL(harvest_5y_area(:,:,l) == val_exp)) harvest_5y_area(:,:,l)= zero
    ENDDO



    ! The routines don't like to store logical variables.  So instead
    ! we create a real array.  If the value of the real array is one,
    ! we assign a value of TRUE to the logical array.  Anything
    ! else is .FALSE.
    r_replant(:,:) = val_exp
    var_name = 'lpft_replant'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ABS(r_replant(ipts,ivm) - un) .LT. 0.001)THEN
             lpft_replant(ipts,ivm)=.TRUE.
          ELSE
             lpft_replant(ipts,ivm)=.FALSE.
          ENDIF
       ENDDO
    ENDDO
     
    ! Read assim_param from restart file. The initialization of assim_param will 
    ! be done in stomate_var_init if the variable is not in the restart file.
    assim_param(:,:,:)  = val_exp
    var_name = 'assim_param'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, npco2, itime, &
         &     .TRUE., assim_param, 'gather', nbp_glo, index_g)

 
    light_tran_to_level_season(:,:,:) = val_exp 
    var_name = 'light_tran_season' 
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevels_tot, itime, & 
         &     .TRUE., light_tran_to_level_season, 'gather', nbp_glo, index_g) 
    IF (ALL(light_tran_to_level_season(:,:,:) == val_exp)) light_tran_to_level_season = zero

    IF (printlev >= 4) WRITE(numout,*) 'Leaving readrestart'
    !-----------------------
  END SUBROUTINE readstart


!! ================================================================================================================================
!! SUBROUTINE   : writerestart
!!
!>\BRIEF          Write all variables for stomate from restart file. 
!!
!! DESCRIPTION  : Write all variables for stomate from restart file. 
!!                
!! \n
!_ ================================================================================================================================

  SUBROUTINE writerestart &
       & (npts, index, dt_days, date, &
       &  adapted, regenerate, moiavail_daily, gdd_init_date, litterhum_daily, &
       &  t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
       &  soilhum_daily, precip_daily, gpp_daily, npp_daily, &
       &  turnover_daily, moiavail_month, moiavail_week, moiavail_growingseason,&
       &  t2m_longterm, tau_longterm, t2m_month, t2m_week, &
       &  tsoil_month, soilhum_month, fireindex, firelitter, &
       &  maxmoiavail_lastyear, maxmoiavail_thisyear, &
       &  minmoiavail_lastyear, minmoiavail_thisyear, &
       &  maxgppweek_lastyear, maxgppweek_thisyear, &
       &  gdd0_lastyear, gdd0_thisyear, precip_lastyear, precip_thisyear, &
       &  gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       &  PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
       &  maxfpc_lastyear, maxfpc_thisyear, &
       &  turnover_longterm, gpp_week, resp_maint_part, &
       &  leaf_age, leaf_frac, plant_status, when_growthinit, age, &
       &  resp_hetero, resp_maint, resp_growth, co2_fire, atm_to_bm, &
       &  veget_lastlight, everywhere, need_adjacent, RIP_time, &
       &  time_hum_min, hum_min_dormance, &
       &  litter, dead_leaves, &
       &  som, lignin_struc, lignin_wood, turnover_time, &
       &  prod_s, prod_m, prod_l, &
       &  flux_s, flux_m, flux_l, flux_prod_s, flux_prod_m, &
       &  flux_prod_l, bm_to_litter, carb_mass_total, &
       &  Tseason, Tseason_length, Tseason_tmp, & 
       &  Tmin_spring_time, onset_date, &
       &  global_years, ok_equilibrium, nbp_accu, nbp_flux, &
       &  nbp_accu_pool, nbp_pool, nbp_pool_start, &
       &  MatrixV, VectorU, previous_stock, current_stock, &
       &  assim_param, CN_som_litter_longterm, &
       &  tau_CN_longterm, KF, k_latosa_adapt, &
       &  rue_longterm, cn_leaf_min_season, nstress_season, soil_n_min, p_O2, bact, &
       &  circ_class_biomass, circ_class_n, store_sum_delta_ba, &
       &  forest_managed, forest_managed_lastyear, &
       &  species_change_map, fm_change_map, lpft_replant, lai_per_level, &
       &  laieff_fit, wstress_season, wstress_month,&
       &  age_stand, rotation_n, last_cut, mai, pai, &
       &  previous_wood_volume, mai_count, coppice_dens, &
       &  light_tran_to_level_season, harvest_5y_area)


    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in) :: npts
    ! Indices of the points on the map
    INTEGER(i_std),DIMENSION(npts),INTENT(in) :: index
    ! time step of STOMATE in days
    REAL(r_std),INTENT(in) :: dt_days
    ! date (d)
    INTEGER(i_std),INTENT(in) :: date
    ! Winter too cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: adapted
    ! Winter sufficiently cold? between 0 and 1
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: regenerate
    ! daily moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_daily
    ! gdd init date
    REAL(r_std),DIMENSION(npts,2),INTENT(in) :: gdd_init_date
    ! daily litter humidity
    REAL(r_std),DIMENSION(npts),INTENT(in) :: litterhum_daily
    ! daily 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_daily
    ! daily minimum 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_min_daily
    ! daily surface temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: tsurf_daily
    ! daily soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: tsoil_daily
    ! daily soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: soilhum_daily
    ! daily precipitations (mm/day) (for phenology)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_daily
    ! daily gross primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gpp_daily
    ! daily net primary productivity (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: npp_daily
    ! daily turnover rates (gC/m**2/day)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in) :: turnover_daily
    ! "monthly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_month
    ! "weekly" moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_week
    ! mean growing season moisture availability 
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: moiavail_growingseason
    ! "long term" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_longterm
    ! "tau_longterm"
    REAL(r_std), INTENT(IN)             :: tau_longterm
    ! "monthly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_month
    ! "seasonal" 2 meter temperatures (K) 
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason_length
    ! temporary variable to calculate Tseason
    REAL(r_std),DIMENSION(npts),INTENT(in)      :: Tseason_tmp
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: Tmin_spring_time
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in)  :: onset_date
!!$    LOGICAL,DIMENSION(npts,nvm),INTENT(in)      :: begin_leaves

    ! "weekly" 2 meter temperatures (K)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: t2m_week
    ! "monthly" soil temperatures (K)
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: tsoil_month
    ! "monthly" soil humidity
    REAL(r_std),DIMENSION(npts,nbdl),INTENT(in) :: soilhum_month
    ! Probability of fire
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: fireindex
    ! Longer term total litter above the ground, gC/m**2 of ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: firelitter
    ! last year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxmoiavail_lastyear
    ! this year's maximum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxmoiavail_thisyear
    ! last year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: minmoiavail_lastyear
    ! this year's minimum moisture availability
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: minmoiavail_thisyear
    ! last year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxgppweek_lastyear
    ! this year's maximum weekly GPP
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxgppweek_thisyear
    ! last year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(in) :: gdd0_lastyear
    ! this year's annual GDD0
    REAL(r_std),DIMENSION(npts),INTENT(in) :: gdd0_thisyear
    ! last year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_lastyear
    ! this year's annual precipitation (mm/year)
    REAL(r_std),DIMENSION(npts),INTENT(in) :: precip_thisyear
    ! growing degree days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_m5_dormance
    ! growing degree days, from begin of season (crops)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_from_growthinit
    ! growing degree days since midwinter (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gdd_midwinter
    ! number of chilling days since leaves were lost (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: ncd_dormance
    ! number of growing days, threshold -5 deg C (for phenology)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: ngd_minus5
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs)
    LOGICAL,DIMENSION(npts,nvm),INTENT(in) :: PFTpresent
    ! "long term" net primary productivity (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: npp_longterm
    ! last year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: lm_lastyearmax
    ! this year's maximum leaf mass, for each PFT (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: lm_thisyearmax
    ! last year's maximum fpc for each natural PFT, on ground
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxfpc_lastyear
    ! this year's maximum fpc for each PFT,
    ! on *total* ground (see stomate_season)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: maxfpc_thisyear
    ! "long term" turnover rate (gC/m**2/year)
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in) :: turnover_longterm
    ! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: gpp_week
    ! maintenance respiration (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nparts),INTENT(in) :: resp_maint_part
    ! leaf age (days)
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(in) :: leaf_age
    ! fraction of leaves in leaf age class
    REAL(r_std),DIMENSION(npts,nvm,nleafages),INTENT(in) :: leaf_frac
    ! Growth and phenological status of the plant
    ! The different stati are defined in constantes
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: plant_status
    ! how many days ago was the beginning of the growing season
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: when_growthinit
    ! mean age (years)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: age
    ! heterotrophic respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_hetero
    ! maintenance respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_maint
    ! growth respiration (gC/day/m**2)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: resp_growth
    ! carbon emitted into the atmosphere by fire (living and dead biomass)
    ! (in gC/m**2/time step)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: co2_fire
    ! biomass taken from the atmosphere (gC or gN/(m**2 of total ground)/day)
    REAL(r_std),DIMENSION(npts,nvm,nelements),INTENT(in) :: atm_to_bm
    ! vegetation fractions (on ground) after last light competition
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: veget_lastlight
    ! is the PFT everywhere in the grid box or very localized
    ! (after its introduction)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: everywhere
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box?
    LOGICAL,DIMENSION(npts,nvm),INTENT(in) :: need_adjacent
    ! How much time ago was the PFT eliminated for the last time (y)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: RIP_time
    ! time elapsed since strongest moisture availability (d)
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: time_hum_min
    ! minimum moisture during dormance
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: hum_min_dormance
    ! fraction of litter above the ground belonging to different PFTs
    REAL(r_std),DIMENSION(npts,nlitt,nvm,nlevs,nelements),INTENT(in) :: litter
    ! dead leaves on ground, per PFT, metabolic and structural,
    ! in gC/(m**2 of ground)
    REAL(r_std),DIMENSION(npts,nvm,nlitt),INTENT(in) :: dead_leaves
    ! SOM pool: active, slow, or passive, (gC(orN)/m**2)
    REAL(r_std),DIMENSION(npts,ncarb,nvm,nelements),INTENT(in) :: som
    ! ratio Lignine/Carbon in structural litter, above and below ground, (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(in) :: lignin_struc
    ! turnover_time of leaves
    ! ratio Lignine/Carbon in woody litter, above and below ground, (gC/m**2)
    REAL(r_std),DIMENSION(npts,nvm,nlevs),INTENT(in) :: lignin_wood
    ! turnover_time of leaves
    REAL(r_std),DIMENSION(npts,nvm),INTENT(in) :: turnover_time

    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: rotation_n              !! Rotation number (number of rotation 
                                                                                                  !! since pft is managed)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: last_cut                !! Years since last thinning (years)
    

    ! For Spinup matrix resolution
    INTEGER(i_std), INTENT(in)                   :: global_years   
    LOGICAL, DIMENSION(npts), INTENT(in)         :: ok_equilibrium
    REAL(r_std), DIMENSION(npts), INTENT(in)     :: nbp_accu        !! Accumulated Net Biospheric Production over the year 
    REAL(r_std), DIMENSION(npts), INTENT(in)     :: nbp_flux        !! Net Biospheric Production over the year 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in) :: nbp_accu_pool   !! Pool-based Accumulated Net Biospheric Production over the year 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in) :: nbp_pool        !! Pool-based Net Biospheric Production over the year 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in) :: nbp_pool_start !! NBP atprevious time step
    !-
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(in) :: MatrixV
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in) :: VectorU
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in) :: previous_stock
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in) :: current_stock
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in) :: CN_som_litter_longterm
    REAL(r_std), INTENT(in)                              :: tau_CN_longterm  !! Counter used for calculating the longterm CN ratio of SOM and litter pools (seconds)
    REAL(r_std), DIMENSION(npts,nvm,npco2),   INTENT(in) :: assim_param
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: KF                !! Scaling factor to convert sapwood mass
                                                                              !! into leaf mass (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: k_latosa_adapt    !! Leaf to sapwood area adapted for water  
                                                                              !! stress. Adaptation takes place at the  
                                                                              !! end of the year (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: mai                     !! The mean annual increment
                                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: pai                     !! The period annual increment
                                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                                  !! in a stand for the previous year.
                                                                                                  !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(in)                     :: mai_count               !! The number of times we've
                                                                                                  !! calculated the volume increment
                                                                                                  !! for a stand
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                        :: coppice_dens            !! The density of a coppice at the first
                                                                                                  !! cutting.
                                                                                                  !! @tex $( 1 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: rue_longterm      !! Longterm radiation use efficiency

    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: cn_leaf_min_season    !! Seasonal CN ratio of leaves 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)         :: nstress_season    !! N-related seasonal stress (used for allocation) 
    
    REAL(r_std), DIMENSION(npts,nvm,nnspec), INTENT(in)  :: soil_n_min        !! mineral nitrogen in the soil (gN/m**2)  
                                                                              !! (first index=npts, second index=nvm, third index=nnspec)   
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)        :: p_O2               !! partial pressure of oxygen in the soil (hPa)
                                                                              !! (first index=npts, second index=nvm)
                                                        
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)        :: bact               !! denitrifier biomass (gC/m**2)
                                                                              !! (first index=npts, second index=nvm)
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)                 :: circ_class_n            !! Number of trees in each circumference classes
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), INTENT(in):: circ_class_biomass      !! Biomass components of the model tree within 
                                                                                                  !! each circumference classes (gC ind-1)
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)                 :: store_sum_delta_ba      !! Sum of increase of basal area(m^2) which
                                                                                                  !! doesn't pass through the gap clean
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot), INTENT(in)           :: lai_per_level           !! The amount of LAI in each physical
                                                                                                  !! canopy level.
                                                                                                  !! @tex $( m**2 / m**2 )$ @endtex
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(in)                     :: laieff_fit              !! Fitted parameters for the effective LAI

    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: wstress_season          !! Water stress factor, based on hum_rel_daily
                                                                                                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: wstress_month           !! Water stress factor, based on hum_rel_daily
    REAL(r_std),DIMENSION(npts,nvm,nparts,nelements),INTENT(in)        :: bm_to_litter
    REAL(r_std),DIMENSION(npts),INTENT(in)                             :: carb_mass_total
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: forest_managed          !! forest management flag
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: forest_managed_lastyear !! forest management flag for the previous year
    REAL(r_std), DIMENSION(npts,0:nshort,nelements), INTENT(in)        :: prod_s                  !!
    REAL(r_std), DIMENSION(npts,0:nmedium,nelements), INTENT(in)       :: prod_m                  !!
    REAL(r_std), DIMENSION(npts,0:nlong,nelements), INTENT(in)         :: prod_l                  !!
    REAL(r_std), DIMENSION(npts,nshort,nelements), INTENT(in)          :: flux_s                  !!
    REAL(r_std), DIMENSION(npts,nmedium,nelements), INTENT(in)         :: flux_m                  !!
    REAL(r_std), DIMENSION(npts,nlong,nelements), INTENT(in)           :: flux_l                  !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)                 :: flux_prod_s             !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)                 :: flux_prod_m             !!
    REAL(r_std), DIMENSION(npts,nelements), INTENT(in)                 :: flux_prod_l             !!
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                         :: species_change_map      !! A map which gives the PFT number that each
                                                                                                  !! PFT will be replanted as in case of a clearcut.
                                                                                                  !! (1-nvm,unitless)
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                         :: fm_change_map           !! A map which gives the desired FM strategy when
                                                                                                  !! the PFT will be replanted after a clearcut.
                                                                                                  !! (1-nvm,unitless)
    LOGICAL, DIMENSION(:,:), INTENT(in)                                :: lpft_replant            !! Indicates if this PFT has either died this year
                                                                                                  !! or been clearcut/coppiced.  If it has, it is not
                                                                                                  !! replanted until the end of the year.
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot), INTENT(in)           :: light_tran_to_level_season  !! Mean seasonal fraction of light transmitted to
                                                                                                  !! a given canopy level (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)                          :: harvest_5y_area         !! Total harvested area in the last 5 years (m^{2})

   !! 0.4 Local variables
    ! date, real
    REAL(r_std) :: date_real
    ! PFT exists (equivalent to fpc_max > 0 for natural PFTs), real
    REAL(r_std),DIMENSION(npts,nvm) :: PFTpresent_real
    ! in order for this PFT to be introduced,
    ! does it have to be present in an adjacent grid box? - real
    REAL(r_std),DIMENSION(npts,nvm) :: need_adjacent_real
    ! To store variables names for I/O
    CHARACTER(LEN=80) :: var_name
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: circ_str 
    ! string suffix indicating an index
    CHARACTER(LEN=10) :: part_str
    ! string suffix indicating litter type
    CHARACTER(LEN=4),DIMENSION(nlitt) :: litter_str
    ! string suffix indicating level
    CHARACTER(LEN=2),DIMENSION(nlevs) :: level_str
    ! temporary storage
    REAL(r_std), DIMENSION(npts) :: ok_equilibrium_real    
    REAL(r_std),DIMENSION(1) :: xtmp
    REAL(r_std), DIMENSION(1) :: vartmp  !! temporary variable because restget/restput needs a variable with DIMESION(:)
    ! index
    INTEGER(i_std) :: j,k,l,m
    CHARACTER(LEN=2),DIMENSION(nelements) :: element_str  !! string suffix indicating element
    REAL(r_std), DIMENSION(1) :: temp_global_years
    CHARACTER(LEN=6),DIMENSION(nbpools) :: pools_str
    INTEGER                                                             :: n,ilev,ipts,ivm         !! Indices
    REAL(r_std), DIMENSION(npts,nvm)                                    :: temp_real               !! temporary real to allow restget 
                                                                                                   !! to work on multi-dimensional integers
    REAL(r_std), DIMENSION(npts,nvm)                                    :: r_replant               !! Getting logical values from the restart
                                                                                                   !! is not possible, so this is a temporary
                                                                                                   !! array where 1.0 is TRUE.
    REAL(r_std),DIMENSION(npts,nvm,nlevels_tot,nparams_laieff)          :: temp_array              !! To store structure values for I/O
    CHARACTER(LEN=10)                                                   :: part_str2               !! string suffix indicating an index
    
    CHARACTER(LEN=2), DIMENSION(wind_years)                             :: wyear_str               !! string suffix indicating wind year index


!_ ================================================================================================================================

    IF (printlev >= 3) WRITE(numout,*) 'Entering writerestart'
    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSEIF (l == iwoody) THEN
          litter_str(l) = 'wood'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define litter_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define level_str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
       ELSEIF (l == initrogen) THEN
          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p(3,'stomate_io writerestart','Define element_str','','')
       ENDIF
    ENDDO
    !-
    pools_str(1:nbpools) =(/'str_ab ','str_be ','met_ab ','met_be ','wood_ab','wood_be', & 
         & 'actif  ','slow   ','passif ','surface'/) 
    !-
    IF (is_root_prc) THEN
       CALL ioconf_setatt_p ('UNITS','-')
       CALL ioconf_setatt_p ('LONG_NAME',' ')
    ENDIF
    !-
    ! 2 run control
    !-
    ! 2.2 time step of STOMATE in days
    !-
    IF (is_root_prc) THEN
       var_name = 'dt_days'
       xtmp(1) = dt_days
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
    ENDIF
    !-
    ! 2.3 date
    !-
    IF (is_root_prc) THEN
       var_name = 'date'
       date_real = REAL(date,r_std)
       xtmp(1) = date_real
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
    ENDIF
    !-
    ! 3 daily meteorological variables
    !-
    var_name = 'moiavail_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_init_date'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    2, 1, itime, &
         &              gdd_init_date, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'litterhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                litterhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_min_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_min_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsurf_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                tsurf_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                precip_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'wstress_season'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                wstress_season, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'wstress_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                wstress_month, 'scatter', nbp_glo, index_g)
    !-
    ! 4 productivities
    !-
    var_name = 'gpp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_daily, 'scatter', nbp_glo, index_g)
    !-

    var_name = 'turnover_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, nelements, itime, &
         &                turnover_daily, 'scatter', nbp_glo, index_g)

    !-
    ! 5 monthly meteorological variables
    !-
    var_name = 'moiavail_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'moiavail_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_week, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'moiavail_grow'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_growingseason, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_longterm, 'scatter', nbp_glo, index_g)
    
    IF (is_root_prc) THEN
       var_name='tau_longterm'
       vartmp(1)=tau_longterm
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, vartmp)
    ENDIF
       

    var_name = 't2m_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
                         t2m_month, 'scatter', nbp_glo, index_g)
    

    CALL restput_p (rest_id_stomate, 'Tseason', nbp_glo,    1, 1, itime, &
         Tseason, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tseason_length', nbp_glo,    1, 1, itime, &
         Tseason_length, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tseason_tmp', nbp_glo,    1, 1, itime, &
         Tseason_tmp, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'Tmin_spring_time', nbp_glo, nvm, 1, itime, &
         Tmin_spring_time, 'scatter', nbp_glo, index_g)
    
    CALL restput_p (rest_id_stomate, 'onset_date', nbp_glo, nvm, 1, itime, &
         onset_date, 'scatter', nbp_glo, index_g)
    
    var_name = 't2m_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_week, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_month, 'scatter', nbp_glo, index_g)
    !-
    ! 6 fire probability
    !-
    var_name = 'fireindex'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                fireindex, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'firelitter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                firelitter, 'scatter', nbp_glo, index_g)
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    var_name = 'maxmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 8 maximum "weekly" GPP
    !-
    var_name = 'maxgppweek_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxgppweek_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 9 annual GDD0
    !-
    var_name = 'gdd0_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd0_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 10 annual precipitation
    !-
    var_name = 'precip_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 11 derived "biometeorological" variables
    !-
    var_name = 'gdd_m5_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_m5_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_from_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              gdd_from_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_midwinter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_midwinter, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ncd_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ncd_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ngd_minus5'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ngd_minus5, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'time_hum_min'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                time_hum_min, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'hum_min_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                hum_min_dormance, 'scatter', nbp_glo, index_g)
    !-
    ! 12 Plant status
    !-
    var_name = 'PFTpresent'
    WHERE ( PFTpresent(:,:) )
       PFTpresent_real = un
    ELSEWHERE
       PFTpresent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                PFTpresent_real, 'scatter', nbp_glo, index_g)
    !-
    !var_name = 'ind'
    !CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
    !     &                ind, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'turnover_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                turnover_time, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'adapted'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                adapted, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'regenerate'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                regenerate, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_longterm, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_lastyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_lastyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_thisyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_thisyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'turnover_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, nelements, itime, &
         &                turnover_longterm, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gpp_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_week, 'scatter', nbp_glo, index_g)
    !-
    !var_name = 'biomass'
    !CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, nelements, itime, &
    !     &                   biomass, 'scatter', nbp_glo, index_g)

    !-
    var_name = 'maint_resp'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, itime, &
         &                   resp_maint_part, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'leaf_age'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nleafages, itime, &
         &                  leaf_age, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'leaf_frac'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nleafages, itime, &
         &                   leaf_frac, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lai_per_level'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevels_tot, itime, &
         &                   lai_per_level, 'scatter', nbp_glo, index_g)
    !
    DO ipts=1,npts
       DO ivm=1,nvm
          DO ilev=1,nlevels_tot
             temp_array(ipts,ivm,ilev,1)=laieff_fit(ipts,ivm,ilev)%a
             temp_array(ipts,ivm,ilev,2)=laieff_fit(ipts,ivm,ilev)%b
             temp_array(ipts,ivm,ilev,3)=laieff_fit(ipts,ivm,ilev)%c
             temp_array(ipts,ivm,ilev,4)=laieff_fit(ipts,ivm,ilev)%d
             temp_array(ipts,ivm,ilev,5)=laieff_fit(ipts,ivm,ilev)%e
          ENDDO
       ENDDO
    ENDDO
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       DO n=1,nparams_laieff
          WRITE(part_str2,'(I2)') n
          IF ( n < 10 ) part_str2(1:1) = '0'

          var_name = 'laieff_fit_'//part_str(1:LEN_TRIM(part_str)) // '_' &
               // part_str2(1:LEN_TRIM(part_str2))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                   temp_array(:,:,m,n), 'scatter', nbp_glo, index_g)
       ENDDO
    ENDDO
    !-
    var_name = 'circ_class_n'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, ncirc, itime, &
         &                   circ_class_n, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'store_sum_delta_ba'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, ncirc, itime, &
         &                   store_sum_delta_ba, 'scatter', nbp_glo, index_g)
    !-
    DO m = 1,nelements
       var_name = 'cc_biomass_'//TRIM(element_str(m))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, ncirc, nparts, itime, &
          &         circ_class_biomass(:,:,:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
   !-
    var_name = 'plant_status'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                plant_status, 'scatter', nbp_glo, index_g)
    !- 
    var_name = 'when_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                when_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'age'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                age, 'scatter', nbp_glo, index_g)
    !-
    ! 13 CO2
    !-
    var_name = 'resp_hetero'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_hetero, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_maint'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_maint, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_growth'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_growth, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'co2_fire'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, 1, itime, &
         &                co2_fire, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'atm_to_bm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nelements, itime, &
         &                atm_to_bm, 'scatter', nbp_glo, index_g)

    !-
    ! 14 vegetation distribution after last light competition
    !-
    var_name = 'veget_lastlight'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                veget_lastlight, 'scatter', nbp_glo, index_g)
    !-
    ! 15 establishment criteria
    !-
    var_name = 'everywhere'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                everywhere, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'need_adjacent'
    WHERE (need_adjacent(:,:))
       need_adjacent_real = un
    ELSEWHERE
       need_adjacent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                need_adjacent_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'RIP_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                RIP_time, 'scatter', nbp_glo, index_g)
    !-
    ! 17 litter
    !-
    DO k = 1,nelements
        var_name = 'litter_'//element_str(k)
        CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlitt, nvm, nlevs, itime, &
             &                     litter(:,:,:,:,k), 'scatter', nbp_glo, index_g)
    END DO
    !-
    var_name = 'dead_leaves'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlitt, itime, &
         &                   dead_leaves, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soil_carbon'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ncarb, nvm, itime, &
         &                   som(:,:,:,icarbon), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soil_nitrogen' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, ncarb, nvm, itime, & 
         &                   som(:,:,:,initrogen), 'scatter', nbp_glo, index_g) 
    !- 
    var_name = 'lignin_struc'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevs, itime, &
         &       lignin_struc, 'scatter', nbp_glo, index_g)
    !- 
    var_name = 'lignin_wood'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevs, itime, &
         &       lignin_wood, 'scatter', nbp_glo, index_g)
    !-
    ! 18 land cover change
    !-
    var_name = 'prod_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nshort+1, nelements, itime, &
         &                prod_s, 'scatter', nbp_glo, index_g)

    var_name = 'prod_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nmedium+1, nelements, itime, &
         &                prod_m, 'scatter', nbp_glo, index_g)

    var_name = 'prod_l'   
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlong+1, nelements, itime, &
         &                prod_l, 'scatter', nbp_glo, index_g)

    var_name = 'flux_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nshort, nelements, itime, &
         &                flux_s, 'scatter', nbp_glo, index_g)
    
    var_name = 'flux_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nmedium, nelements, itime, &
         &                flux_m, 'scatter', nbp_glo, index_g)

    var_name = 'flux_l'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlong, nelements, itime, &
         &                flux_l, 'scatter', nbp_glo, index_g)

    var_name = 'flux_prod_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nelements, 1, itime, &
         &              flux_prod_s, 'scatter', nbp_glo, index_g)

    var_name = 'flux_prod_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nelements, 1, itime, &
         &              flux_prod_m, 'scatter', nbp_glo, index_g)
    
    var_name = 'flux_prdo_l'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nelements, 1, itime, &
         &              flux_prod_l, 'scatter', nbp_glo, index_g)

    var_name = 'bm_to_litter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nparts, nelements, itime, &
         &                bm_to_litter, 'scatter', nbp_glo, index_g)

    var_name = 'carb_mass_total'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              carb_mass_total, 'scatter', nbp_glo, index_g)


! put the harvest_5y_area to the restart file
    DO l=1,wind_years
       WRITE(wyear_str(l),'(I2.2)') l
       var_name = 'harvest_area_year_'//wyear_str(l)
       !WRITE(numout,*) 'YC: 20160531: var_name for restart file', var_name
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &     harvest_5y_area(:,:,l), 'scatter', nbp_glo, index_g)
    ENDDO




    !-
    ! 19. Spinup
    !-
    IF (spinup_analytic) THEN

       IF (is_root_prc) THEN
          temp_global_years(1) = REAL(global_years)
          var_name='Global_years'
          CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, temp_global_years)
       ENDIF
       
       var_name = 'nbp_sum'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_accu, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_flux'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_flux, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_accu_pool'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              nbp_accu_pool, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_pool'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              nbp_pool, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_pool_start'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              nbp_pool_start, 'scatter', nbp_glo, index_g)

       var_name = 'ok_equilibrium'
       WHERE(ok_equilibrium(:))
          ok_equilibrium_real = un
       ELSEWHERE
          ok_equilibrium_real = zero
       ENDWHERE
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &               ok_equilibrium_real, 'scatter', nbp_glo, index_g)
       
       DO k = 1,nbpools
          DO j = 1,nbpools
             WRITE(part_str,'(I2)') k
             IF (k < 10) part_str(1:1) = '0'
             var_name = 'MatrixV_'//part_str(1:LEN_TRIM(part_str))//'_'//TRIM(pools_str(j))
             CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime,&
                  &                MatrixV(:,:,k,j), 'scatter', nbp_glo,index_g)
          ENDDO
       ENDDO

       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'Vector_U_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                VectorU(:,:,k), 'scatter', nbp_glo, index_g)
       ENDDO

       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'previous_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                previous_stock(:,:,k), 'scatter', nbp_glo,index_g)
       ENDDO
       
       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'current_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                current_stock(:,:,k), 'scatter', nbp_glo,index_g)
       ENDDO

       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'CN_longterm_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                CN_som_litter_longterm(:,:,k), 'scatter',nbp_glo, index_g)
       ENDDO

       IF (is_root_prc) THEN
          var_name='tau_CN_longterm'
          vartmp(1)=tau_CN_longterm
          CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, vartmp)
       ENDIF
            
    ENDIF !(spinup_analytic)

    !-
    var_name = 'KF'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              KF(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'k_latosa_adapt'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              k_latosa_adapt(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'rue_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              rue_longterm(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'cn_leaf_min_season' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              cn_leaf_min_season(:,:), 'scatter', nbp_glo, index_g) 
    !- 
    var_name = 'nstress_season' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              nstress_season(:,:), 'scatter', nbp_glo, index_g) 
    !- 
    var_name = 'soil_n_min' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nnspec, itime, & 
         &              soil_n_min, 'scatter', nbp_glo, index_g) 
    !- 
    var_name = 'p_O2' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              p_O2(:,:), 'scatter', nbp_glo, index_g) 
    !- 
    var_name = 'bact' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              bact(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'age_stand'
    temp_real = REAL(age_stand,r_std)
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'rotation_n'
    temp_real = REAL(rotation_n,r_std)  
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'last_cut'
    temp_real = REAL(last_cut,r_std)
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'mai'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              mai, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'pai'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              pai, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'coppice_dens'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              coppice_dens, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'previous_wood_volume'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              previous_wood_volume, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'mai_count'
    temp_real = REAL(mai_count,r_std)
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'fm_lyear'
    temp_real = REAL(forest_managed_lastyear,r_std)
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    
    var_name = 'fm_current'
    temp_real = REAL(forest_managed,r_std)
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              temp_real, 'scatter', nbp_glo, index_g)
    !- 
    ! For these next three variables, we have to convert to a real array first.
    r_replant(:,:)=REAL(species_change_map(:,:),r_std)
    var_name = 'sc_map'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)
    
    r_replant(:,:)=REAL(fm_change_map(:,:),r_std)
    var_name = 'fmc_map'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)

    DO ipts=1,npts
       DO ivm=1,nvm
          IF(lpft_replant(ipts,ivm))THEN
             r_replant(ipts,ivm)=un
          ELSE
             r_replant(ipts,ivm)=zero
          ENDIF
       END DO
    END DO
    var_name = 'lpft_replant'
    CALL restput_p(rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)

    var_name = 'assim_param'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, npco2, itime, &
         &                assim_param, 'scatter', nbp_glo, index_g)

    ! 21 Seasonal mean transmitted light for recruitment in DOFOCO  
    var_name = 'light_tran_season' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, nlevels_tot, itime, & 
 	&                   light_tran_to_level_season, 'scatter', nbp_glo, index_g)


    IF (printlev >= 4) WRITE(numout,*) 'Leaving writerestart'
    !--------------------------

  END SUBROUTINE writerestart
  !-
  !===
  !-
END MODULE stomate_io
